"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = doesQueryContain;

var _getPotentialAncestors = _interopRequireDefault(require("./getPotentialAncestors"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function doesQueryContain(document, types, typename, data, ids, idField = 'id') {
  const targetType = types[typename];
  if (!targetType) throw new Error(`type not found: ${typename}`);
  const potentialAncestors = (0, _getPotentialAncestors.default)(targetType);

  function doesNodeContain(node, data, type) {
    if (type === targetType) {
      if (!ids) return true;
      return data && ids.has(data[idField]);
    }

    if (!type.name) return false;
    const ancestorEntry = potentialAncestors.get(type);
    if (!ancestorEntry) return false;
    const {
      selectionSet
    } = node;

    if (selectionSet) {
      for (let selection of selectionSet.selections) {
        const alias = selection.alias;
        const name = selection.name;
        if (!name) continue;
        if (!ancestorEntry.fields.has(name.value)) continue;
        const {
          fields
        } = type;
        if (!fields) continue;
        let field = fields[name.value];
        if (!field) continue;
        let innerType = field.type;
        let list = false;

        while (innerType.ofType) {
          if (innerType.kind === 'LIST') list = true;
          innerType = innerType.ofType;
        }

        let innerData = data ? data[alias ? alias.value : name.value] : null;
        if (ids && innerData == null) continue;

        if (ids && list) {
          if (!Array.isArray(innerData)) continue;

          for (let element of innerData) {
            if (doesNodeContain(selection, element, innerType)) {
              return true;
            }
          }
        } else {
          if (doesNodeContain(selection, innerData, innerType)) {
            return true;
          }
        }
      }
    }

    return false;
  }

  for (let node of document.definitions) {
    if (doesNodeContain(node, data, types.Query)) {
      return true;
    }
  }

  return false;
}