"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = getPotentialAncestors;
const cache = new Map();

function getPotentialAncestors(type) {
  const added = new Set();
  const cached = cache.get(type);
  if (cached) return cached;
  const ancestors = new Map();

  function addType(type) {
    if (added.has(type)) return;
    added.add(type);
    const {
      parents
    } = type;

    if (parents) {
      for (let field of parents) {
        const {
          name,
          parent
        } = field;
        let ancestor = ancestors.get(parent);

        if (!ancestor) {
          ancestor = {
            fields: new Set()
          };
          ancestors.set(parent, ancestor);
        }

        ancestor.fields.add(name);
        addType(parent);
      }
    }
  }

  addType(type);
  cache.set(type, ancestors);
  return ancestors;
}