"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.linkTypes = linkTypes;
exports.default = getSchemaTypesOnce;

var _typesQuery = _interopRequireDefault(require("./typesQuery"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

let promise;

function convertRawField({
  name,
  type
}) {
  return {
    name,
    type: convertRawType(type)
  };
}

function convertRawFields(fields) {
  const convertedFields = {};

  for (let field of fields) {
    convertedFields[field.name] = convertRawField(field);
  }

  return convertedFields;
}

function convertRawType({
  name,
  kind,
  ofType,
  fields
}) {
  return {
    name,
    kind,
    ofType: ofType ? convertRawType(ofType) : null,
    fields: fields ? convertRawFields(fields) : null
  };
}

function linkTypes(rawTypes) {
  const types = {};

  for (let rawType of rawTypes) {
    const {
      name
    } = rawType;

    if (name) {
      types[name] = convertRawType(rawType);
    }
  }

  function resolveType(type, parent) {
    const {
      name,
      ofType
    } = type;
    if (name && types[name]) type = types[name];
    if (ofType) type.ofType = resolveType(ofType, parent);

    if (parent) {
      let {
        parents
      } = type;
      if (!parents) type.parents = parents = [];
      parents.push(parent);
    }

    return type;
  }

  for (let name in types) {
    const type = types[name];
    const {
      fields
    } = type;

    if (fields) {
      for (let name in fields) {
        const field = fields[name];
        field.type = resolveType(field.type, field);
        field.parent = type;
      }
    }
  }

  return types;
}

async function getSchemaTypes(client) {
  const {
    data: {
      __schema: {
        types
      }
    }
  } = await client.query({
    query: _typesQuery.default
  });
  return linkTypes(types);
}

function getSchemaTypesOnce(client) {
  if (!promise) promise = getSchemaTypes(client);
  return promise;
}