"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = refetch;

var _apolloClient = require("apollo-client");

var _getSchemaTypes = _interopRequireDefault(require("./getSchemaTypes"));

var _doesQueryContain = _interopRequireDefault(require("./doesQueryContain"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function normalizeIds(ids) {
  if (ids instanceof Set) return ids;
  if (Array.isArray(ids)) return new Set(ids);
  return new Set([ids]);
}

function every(array, predicate) {
  for (let elem of array) {
    if (!predicate(elem)) return false;
  }

  return true;
}

async function refetch(client, typenameOrTerms, ids, idField) {
  if (!(client instanceof _apolloClient.ApolloClient)) throw new Error(`client must be an ApolloClient, instead got: ${String(client)}`);
  const types = await (0, _getSchemaTypes.default)(client);
  let terms;

  if (typeof typenameOrTerms === 'string') {
    terms = [[typenameOrTerms, ids, idField]];
  } else if (Array.isArray(typenameOrTerms)) {
    terms = typenameOrTerms;
  } else {
    throw new Error(`invalid typename or terms: ${typenameOrTerms}`);
  }

  const {
    queryManager: {
      queries
    }
  } = client;
  let promises = [];

  for (let query of queries.values()) {
    const {
      document,
      observableQuery
    } = query;
    if (!observableQuery) continue;
    let data;
    const currentResult = observableQuery.currentResult();
    if (currentResult) data = currentResult.data;

    if (every(terms, ([typename, ids, idField]) => (0, _doesQueryContain.default)(document, types, typename, data, ids != null ? normalizeIds(ids) : null, idField || 'id'))) {
      promises.push(observableQuery.refetch());
    }
  }

  await promises;
}